define([
    'jquery',
    'underscore',
    'knockout',
    'mage/translate',
    'Magento_PageBuilder/js/events',
    'Magento_PageBuilder/js/content-type/preview-collection',
    'Magento_PageBuilder/js/content-type-factory',
    'Magento_PageBuilder/js/config',
    'Magento_PageBuilder/js/content-type-menu/option'
], function ($, _, ko, $t, events, PreviewCollection, createContentType, pageBuilderConfig, option) {
    'use strict';

    /**
     * @param parent
     * @param config
     * @param stageId
     * @constructor
     */
    function Preview(parent, config, stageId) {
        PreviewCollection.call(this, parent, config, stageId);
    }

    Preview.prototype = Object.create(PreviewCollection.prototype);

    /**
     * Root element
     */
    Preview.prototype.element = null;

    Preview.prototype.initializeAccordionWidget = _.debounce(function () {
        if (this.element) {

            var ppfaqItems = $(this.element).find('.pagebuilder-ppfaq-item');

            ppfaqItems.each(function() {
                $(this).find('div[data-role="content"]').hide();
            });

            $('.pagebuilder-ppfaq-item').unbind('click').on('click', 'h3', function() {
                var content = $(this).next();
                content.slideToggle();
            })

        }
    }, 10);

    /**
     * Bind events to add empty ppfaq item when ppfaq added and reinitialize accordion when ppfaq item added
     */
    Preview.prototype.bindEvents = function bindEvents() {
        var self = this;

        PreviewCollection.prototype.bindEvents.call(this);

        events.on("ppfaq:dropAfter", function (args) {
            if (args.id === self.contentType.id && self.contentType.children().length === 0) {
                self.addppfaqItem();
            }
        });

        events.on("ppfaq-item:renderAfter", (args) => {
            if (args.contentType.parentContentType.id === self.contentType.id) {
                this.initializeAccordionWidget();
            }

        });

    };

    /**
     * Add ppfaq item
     */
    Preview.prototype.addppfaqItem = function () {
        var self = this;
        createContentType(
            pageBuilderConfig.getContentTypeConfig("ppfaq-item"),
            this.contentType,
            this.contentType.stageId,
            {
                question: $t("Question ") + (self.contentType.children().length + 1),
                answer: $t("Edit answer here.")
            }
        ).then(function (container) {
            self.contentType.addChild(container);
        });
    };

    /**
     * Return content menu options
     *
     * @returns {object}
     */
    Preview.prototype.retrieveOptions = function () {
        var self = this;
        var options = PreviewCollection.prototype.retrieveOptions.call(this);

        options.add = new option({
            preview: this,
            icon: "<i class='icon-pagebuilder-add'></i>",
            title: "Add",
            action: self.addppfaqItem,
            classes: ["add-child"],
            sort: 10
        });
        return options;
    };

    /**
     * Set root element
     *
     * @returns {void}
     */
    Preview.prototype.afterRender = function (element) {
        this.element = element;
    };

    /**
     * Check if content type is container
     *
     * @returns {boolean}
     */
    Preview.prototype.isContainer = function () {
        return true;
    };

    return Preview;
});
